from pyRepetUnit.commons.seq.BioseqDB import BioseqDB

TE_SIGNIF_CATEGORY_ORF_BOUNDS = [1000, 3000, 100000]
BLAST_ID_WEIGHT = ((50,100,1), (40,50,0.9), (30,40,0.8), (0,30,0.7))
PROFILES_SCORE_WEIGHT = [0.8, 1]
MAX_CI = 100

class Environment(object):
    
    def __init__(self, consensusFileName, iDb, projectName = "", iConfig = None, verbose = 0):
        #TODO: Bioseq instance : sequence unused, but length needed => create an other class (name + length)
        iBioseqDB = BioseqDB(consensusFileName)
        self._iBioseqDB = iBioseqDB
        self._iDb = iDb
        self._TESignifCategoryORFBounds = TE_SIGNIF_CATEGORY_ORF_BOUNDS
        self._blastIdWeight = BLAST_ID_WEIGHT
        self._profilesScoreWeight = PROFILES_SCORE_WEIGHT
        
        self._maxProfilesEvalue = 1e-3
        self._maxBlastnEvalue = 1e-3
        self._minBlastIndex = 5
        self._minProfilesIndex = 19
        self._minSsrCoverage = 0.75
        
        self._blackBoardDemands = []
        self._blackBoardResults = []
        self._maxCI = MAX_CI
        self._projectName = projectName
        self._iConfig = iConfig
        self._verbosity = verbose
        
        self.setTablesList()
    
    def setTablesList(self):
        self._lTables = self._iDb.getTableListFromPattern(self._projectName + "%")
        
    def getBioseqDBInstance(self):
        return self._iBioseqDB
    
    def getTESignifCategoryORFBounds(self):
        return self._TESignifCategoryORFBounds
    
    def getBlastIdWeight(self):
        return self._blastIdWeight
    
    def getProfilesScoreWeight(self):
        return self._profilesScoreWeight
    
    def getMaxProfilesEvalue(self):
        return self._maxProfilesEvalue
    
    def getMaxBlastnEvalue(self):
        return self._maxBlastnEvalue
    
    def getMinBlastIndex(self):
        return self._minBlastIndex
    
    def getMinProfilesIndex(self):
        return self._minProfilesIndex
    
    def getMinSSRCoverage(self):
        return self._minSsrCoverage
    
    def getMaxCI(self):
        return self._maxCI
    
    def getConsensusNameList(self):
        return self._iBioseqDB.getHeaderList()
    
    def getBlackBoardResults(self):
        return self._blackBoardResults
   
    def getBlackBoardDemands(self):
        return self._blackBoardDemands

    def getDbInstance(self):
        return self._iDb
    
    def getProjectName(self):
        return self._projectName
    
    def getTablesList(self):
        return self._lTables
    
    def getResultsOfOneAgent(self, agentName):
        listOfResultForSpecifiedAgent = []
        for result in self._blackBoardResults:
            if result.demand.agent == agentName:
                listOfResultForSpecifiedAgent.append(result)
        return listOfResultForSpecifiedAgent
    
    def addDemandToBlackBoard(self, demand):
        self._blackBoardDemands.append(demand)
    
    def addResultToBlackBoard(self, result):
        self._blackBoardResults.append(result)
   
    def isDemandOnBlackBoardDemands(self, demand):
        return demand in self._blackBoardDemands
   
    def isDemandOnBlackBoardResults(self, demand):
        for res in self._blackBoardResults:
            if res.demand == demand:
                return True
        return False
     
    def isResultOnBlackBoard(self, result):
        return result in self._blackBoardResults
        
    def isConsensusInEnvironment(self, consensusName):
        return consensusName in self.getConsensusNameList()
    
    def displayBlackBoardResult(self):
        for res in self._blackBoardResults:
            if hasattr(res,"getResult"):
                if isinstance(res.getResult(),list):
                    for item in res.getResult():
                        print item
                else:
                    print res.getResult()
            else:
                print(res)
            
    def cleanBlackBoardDemands(self):
        self._blackBoardDemands = []
            
    def cleanBlackBoardResults(self):
        self._blackBoardResults = []
        
    def getResultAccordingToDemand(self, iDemand):
        listOfResults = self.getBlackBoardResults()
        for result in listOfResults:
            if result.demand == iDemand:
                return result
        raise Exception("The result corresponding at %s-%s is not found on blackboard\n" % (iDemand.agent, iDemand.consensusName))
    
    def getConsensusLength(self, consensusName):
        return self._iBioseqDB.getSeqLength(consensusName)
    
    def _createAttrNameFromConfigKey(self, configKey):
        attributeName = configKey.replace("_", " ")
        attributeName = attributeName.title()
        attributeName = attributeName.replace(" ","") 
        attributeName = "_" + attributeName[0].lower() + attributeName[1:]
        return attributeName
    
    def setAttributesFromConfig(self):
        if self._projectName == "":
            self._projectName = self._iConfig.get("project", "project_name")
        sectionName = "classif_consensus"
        for optionName in self._iConfig.options(sectionName):   
            attributeName = self._createAttrNameFromConfigKey(optionName)
            if hasattr(self, attributeName) and self._iConfig.get(sectionName, optionName) != "":
                try: 
                    setattr(self, attributeName, float(self._iConfig.get(sectionName, optionName)))
                except:   
                    print "Error setting the parameter %s" % attributeName
            elif self._verbosity > 0:
                print "The parameter %s does not exist" % attributeName