from pyRepetUnit.commons.sql.TablePathAdaptator import TablePathAdaptator
from PASTEC.DemandAndResult import DemandAndResult
from PASTEC.BlasterMatch import BlasterMatch
from PASTEC.utils.AbstractAnalyseAgent import AbstractAnalyseAgent
import decimal

## This agent search in the database the best blastn match against host genes
#
class HG_BLRn_Agent(AbstractAnalyseAgent):
    
    SUFFIX_TABLE_NAME_LIST = ["_HG_BLRn_path"] 
    
    def __init__(self, env):
        self._dAgentName2AnalyserInstance = {}
        AbstractAnalyseAgent.__init__(self, env)
        self._bestMatchName = None
        self._bestMatchIdentity = None
        self._coverage = None
            
    def analyse(self, demand, lOfFeature):
        lPathSorted = self._sortLPathBySubjectNameAndQueryStartAndByQueryEnd(lOfFeature)
        consensusLength = self.askForConsensusLength(demand.consensusName)
        iBlasterMatch = None
        if lPathSorted != []:
            iPath = self._searchBestMatch(lPathSorted)
            self._bestMatchName =  iPath.range_subject.seqname
            self._bestMatchIdentity = iPath.identity
            self._setCoverage(consensusLength, iPath)
            iBlasterMatch = BlasterMatch(self._bestMatchName, self._coverage, identity = self._bestMatchIdentity)
        return DemandAndResult(demand, iBlasterMatch)
    
    def getSuffixTableNameList(self):
        return self.SUFFIX_TABLE_NAME_LIST

    def getListOfFeaturesToAnalyse(self, consensusName):
        tpA = TablePathAdaptator(self._db, self._tableName)
        lPath = tpA.getPathListFromQuery(consensusName)
        return lPath

    def _setCoverage(self, consensusLength, bestMatch):
        length_match = bestMatch.range_query.end - bestMatch.range_query.start + 1
        decimal.getcontext().prec = 4 
        self._coverage = float((decimal.Decimal(length_match) * 100) / decimal.Decimal(consensusLength)) 

    def _searchBestMatch(self, list_path):
        bestMatch = list_path[0]
        max_score1 = list_path[0].score
        evalue1 = list_path[0].e_value
        for match in list_path:
            if match.score > max_score1:
                max_score1 = match.score
                evalue1 = match.e_value
                bestMatch = match
            #TODO: elif ? no cutoff for E value ?
            if match.score == max_score1 and match.e_value > evalue1:
                evalue1 = match.e_value
                bestMatch = match
        return bestMatch
    
    def _sortLPathBySubjectNameAndQueryStartAndByQueryEnd(self, lPath):
        return sorted( lPath, key=lambda iPath: ( iPath.range_subject.seqname, iPath.range_query.start, iPath.range_query.end ))
