from PASTEC.ClassifierResult import ClassifierResult
from PASTEC.utils.AbstractAdaptatorAgent import AbstractAdaptatorAgent

class HG_profilesAdaptatorAgent(AbstractAdaptatorAgent):

    CI_MAX = 1
    #TODO: set in config
    MIN_COVERAGE = 0.75
    INIT_CLASSIF = "!PotentialHostGene"

    def __init__(self, env):
        AbstractAdaptatorAgent.__init__(self, env)
        self._listOfKnownAgents = ["Profiles_Agent"]
    
    def getAgentName(self):
        return "PotentialHostGene"
    
    ## Set conclusionsMatch
    #  
    def setConclusionsMatch(self, conclusionsMatch):
        self._conclusionsMatch = conclusionsMatch
        
    def getConclusionsMatch(self):
        return self._conclusionsMatch

    ## Three main step : make demands, get results and decide
    #
    def live(self):
        for demand in self._environment.getBlackBoardDemands():
            if self.acceptDemand(demand):
                self._consensusName = demand.consensusName 
                self._conclusionsMatch = []
                self._makeDemands()               
                
                self.getAnalysesResults()
                classification, normalizedIC = self.decide()
                message = self.generateMessage(classification, normalizedIC)
                
                finalResult = ClassifierResult(demand, message)
                self._environment.addResultToBlackBoard(finalResult)
            
    ## Give the classification and the confidence index from all analysis conclusions
    #
    # @return classification, normalized confidence index
    #
    def decide(self):
        classification = self.INIT_CLASSIF
        self._CI = 0
        for conclusion in self._conclusionsMatch:
            classification = self._considerProfilesIfTheyAreDiscriminant(conclusion, classification)
        normalizedIC = self._normalizeIC()
        return classification, normalizedIC
    
    def makeAppropriateConclusionsForProfiles_Agent( self, iProfilesResult ):
        if iProfilesResult != None:
            self._conclusionsMatch.extend(iProfilesResult.listProfilesDetailed)                
    
    def _considerProfilesIfTheyAreDiscriminant(self, conclusion, classification):
        if conclusion.has_key('coverage') and conclusion['coverage'] > self.MIN_COVERAGE:
            classification = self.getAgentName()
            self._CI += 1                 
        return classification
    
    def _fillDictOfEvidence(self):
        dictOfEvidence = {}
        dictOfProfiles = {}
        for conclusion in self._conclusionsMatch:
            if conclusion.has_key('name'):
                dictOfProfiles[conclusion['name']] = conclusion['coverage']
        if dictOfProfiles != {}:
            dictOfEvidence['Other_profiles'] = dictOfProfiles
        return dictOfEvidence
