from PASTEC.utils.AbstractAdaptatorAgent import AbstractAdaptatorAgent
import re

class SSRAdaptatorAgent(AbstractAdaptatorAgent):

    CI_MAX = 1
    INIT_CLASSIF = "!SSR"
    CATEGORY_LENGTH_TE = ["<100bps", ">100bps"]
    CATEGORY_LENGTH_BOUND = [100]

    def __init__(self, env):
        AbstractAdaptatorAgent.__init__(self, env)
        #TODO: Size_Agent unused, but to specify in list to call makeAppropriateConclusionsForSize_Agent()...
        self._listOfKnownAgents = ["Size_Agent", "SSRtrf_Agent"]
    
    def getAgentName(self):
        return "SSR"

    ## Set conclusionsSearch
    #  
    def setConclusionsSearch(self, conclusionsSearch):
        self._conclusionsSearch = conclusionsSearch
        
    def getConclusionsSearch(self):
        return self._conclusionsSearch

    ## Give the classification and the confidence index from all analysis conclusions
    #
    # @return classification, normalized confidence index
    #
    def decide(self):
        classification = self.INIT_CLASSIF
        self._CI = 0
        for conclusion in self._conclusionsSearch:
            classification = self._considerLengthAndSSRCoverage(conclusion, classification)
        normalizedIC = self._normalizeIC()
        return classification, normalizedIC
                
    def makeAppropriateConclusionsForSize_Agent(self, iUnusedVar):
        consensusLength = self._environment.getConsensusLength(self._consensusName)
        lBound = sorted(self.CATEGORY_LENGTH_BOUND)
        msg = "<%sbps" % lBound[0]
        for bound in lBound :
            if consensusLength > bound:
                msg = ">%sbps" % bound
        self._conclusionsSearch.append(msg)
                    
    def makeAppropriateConclusionsForSSRtrf_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != None:
            iSSRCollection = iDemandAndResult.result
            coverage = float(iSSRCollection.getSSRCoverage())
            if coverage > self._environment.getMinSSRCoverage():
                formatedCoverage = "%.2f>%s" % (coverage, self._environment.getMinSSRCoverage())
                self._conclusionsSearch.append(formatedCoverage)
            else:
                formatedCoverage = "%.2f<%s" % (coverage, self._environment.getMinSSRCoverage())
                self._conclusionsSearch.append(formatedCoverage)

    def _considerLengthAndSSRCoverage(self, conclusion, classification):
        if re.match(self.CATEGORY_LENGTH_TE[0], conclusion) or re.match("[0-9.]+>" + str(self._environment.getMinSSRCoverage()), conclusion):
            classification = self.getAgentName()
            self._CI += 1
        return classification
    
    def _fillDictOfEvidence(self):
        dictOfEvidence = {}
        for conclusion in self._conclusionsSearch:
            if re.match("[0-9.]+(>|<)" + str(self._environment.getMinSSRCoverage()), conclusion):
                dictOfEvidence['SSRCoverage'] = conclusion
            if re.match(".*bps", conclusion):
                dictOfEvidence['length'] = conclusion
        return dictOfEvidence
    
    def _initializeConclusions(self):
        self._conclusionsSearch = []