from PASTEC.utils.IAgent import IAgent
from PASTEC.Demand import Demand
from PASTEC.DecisionMakerPerConsensus import DecisionMakerPerConsensus
from PASTEC.OutputWriterPerConsensus import OutputWriterPerConsensus
from PASTEC.utils.AgentFactory import AgentFactory

#TODO: check tables according to config ?
class SuperAgent(IAgent):
 
    def __init__(self, env):
        self._lKnownAgents = ["LINE_Agent", "Helitron_Agent", "rDNAAdaptatorAgent", "HG_BLRnAdaptatorAgent", \
                              "HG_profilesAdaptatorAgent", "SSRAdaptatorAgent", "LTR_Agent", "PLE_Agent", \
                              "DIRS_Agent", "SINE_Agent", "LARD_Agent", "TRIM_Agent", "TIR_Agent", "Maverick_Agent", \
                              "Crypton_Agent", "MITE_Agent", "Strand_Agent", "ClassI_Agent", "ClassII_Agent"]
        self._environment = env
        self._dAgentName2Instance = {}
        
    def createNeededAgentInstance(self):
        for knownAgentName in self._lKnownAgents:
            currentAgentNewInstance = AgentFactory.createAgent(knownAgentName, self._environment)
            currentAgentNewInstance.createNeededAgentInstance() 
            self._dAgentName2Instance [currentAgentNewInstance.__class__.__name__] = currentAgentNewInstance 
        
    def live(self):
        self.makeDemands()
        self._callAgent()
        lConsensusNames = self._environment.getConsensusNameList()
        lResults = self.getAnalysesResults()
        outFileName = "%s.classif" % self._environment.getProjectName()
        outF = open(outFileName, "w")
        dConsensusName2Results = self.createDictOfListOfResultsPerConsensusName(lResults)
        for consensusName in lConsensusNames:
            consensusLength = self._environment.getConsensusLength(consensusName)
            dResults2Write = self.decidePerConsensus(consensusName, dConsensusName2Results[consensusName], consensusLength)
            self.outputWriter(outF, dResults2Write)
        outF.close()

    def makeDemands(self):
        lConsensusName = self._environment.getConsensusNameList()
        for consensusName in lConsensusName:
            for agent in self._lKnownAgents:
                myDemand = Demand(agent, consensusName)
                self._environment.addDemandToBlackBoard(myDemand) 
                
    def getAnalysesResults(self):
        lResults = []
        for agent in self._lKnownAgents:
            if agent != "Strand_Agent":
                lResults.extend(self._environment.getResultsOfOneAgent(agent))
        return lResults    
    
    def createDictOfListOfResultsPerConsensusName(self, lResults):
        lResultsSorted = sorted(lResults, key=lambda k: k.message['consensusName'])
        dConsensusName2Results = {}
        previousConsensusName = ""
        lResultOfConsensusName = []
        for result in lResultsSorted:                        
            currentConsensusName = result.message['consensusName']
            if currentConsensusName != previousConsensusName and lResultsSorted.index(result) != 0:
                dConsensusName2Results[previousConsensusName] = lResultOfConsensusName
                lResultOfConsensusName = []
            lResultOfConsensusName.append(result)
            previousConsensusName = currentConsensusName
            if lResultsSorted.index(result) == len(lResultsSorted) - 1:
                dConsensusName2Results[previousConsensusName] = lResultOfConsensusName
         
        return dConsensusName2Results
    
    def decidePerConsensus(self, consensusName, lResults, consensusLength):  
        decisionMaker = DecisionMakerPerConsensus(self._environment)
        d2Return = decisionMaker.decidePerConsensus(consensusName, lResults, str(consensusLength))
        return d2Return
    
    def outputWriter(self, outF, dResult):
        outputWriter = OutputWriterPerConsensus()
        outputWriter.outputWriter(outF, dResult)
        
    def _callAgent(self):
        for agent in self._lKnownAgents:
            #TODO: REFACTORING !!! first loop on analyzers, close DB, second loop on classifiers...
            if agent == "SSRAdaptatorAgent":
                self._environment._iDb.close()
            self._dAgentName2Instance[agent].live()
