from PASTEC.ProteinDomainMatch import ProteinDomainMatch
from PASTEC.DemandAndResult import DemandAndResult
from PASTEC.Profiles_Agent import Profiles_Agent
from PASTEC.TEProfilesResult import TEProfilesResult
import re

## This agent search in the database the significant profiles and their coverage in TE consensus (TE profiles)
#  It find the frame of profile  
#
# Inherit from Profiles_Agent
#
class TE_Profiles_Agent(Profiles_Agent):
    
    def __init__(self, env):
        Profiles_Agent.__init__(self, env)
        self._filterRuleOnProfileType = self._isOTHERNotInSeqName
        self._dProfilesFrames = None
        
    def createResult(self, demand):
        lProteinDomainMatchs = []
        for dProfile in self._lProfilesDetailed:
            iProteinDomainMatch = ProteinDomainMatch(dProfile["name"], dProfile["coverage"], dProfile["start"], re.sub("Frameshifted", "", dProfile['strand']), self._dProfilesFrames[dProfile["name"]])
            lProteinDomainMatchs.append(iProteinDomainMatch)
        return DemandAndResult(demand, lProteinDomainMatchs)

#TODO: Check if frame is only calculate when it's possible (no frameshifted) before merged same profile ?????
    def calculOfProfilesFrameComparedWithConsensus(self, listProfilesDetailed, currentConsensusLength):
        profilesFrames = {}
        for profilesDetailed in listProfilesDetailed:
            profilesFrames[profilesDetailed['name']] = []
        for profilesDetailed in listProfilesDetailed:
            profilesStrand = re.sub("Frameshifted", "", profilesDetailed['strand'])
            profilesName = profilesDetailed['name']  
            if re.search("Frameshifted", profilesDetailed['strand']): 
                profilesFrames[profilesName].append(0)         
            elif profilesStrand == "+":
                profilesFrames[profilesName].append((profilesDetailed['start'] - 1) % 3 + 1)
            elif profilesStrand == "-":
                profilesFrames[profilesName].append(-((currentConsensusLength - profilesDetailed['start']) % 3 + 1))
        self._dProfilesFrames =  self._getFramesIfAllMatchesOfOneProfilesAreInTheSameFrame(profilesFrames)
        
    def patch(self, iDemandAndResult):
        lProfilesDetailed = []
        dProfilesFrames = {}
        for iProteinDomainMatch in iDemandAndResult.result:
            lProfilesDetailed.append({"name": iProteinDomainMatch.getName(),
                                      "coverage": float(iProteinDomainMatch.getCoverage()),
                                      "start": iProteinDomainMatch.getStart(),
                                      "type": iProteinDomainMatch.getType()
                                      })
            dProfilesFrames[iProteinDomainMatch.getName()] = iProteinDomainMatch.getFrame()
        return TEProfilesResult(iDemandAndResult.demand, lProfilesDetailed, dProfilesFrames)
    
    def _getFramesIfAllMatchesOfOneProfilesAreInTheSameFrame(self, profilesFrames):
        dNewProfilesFrames = {}
        for profilesName in profilesFrames.keys():
            if ((sum(profilesFrames[profilesName]) / len(profilesFrames[profilesName])) == profilesFrames[profilesName][0]):
                dNewProfilesFrames[profilesName] = profilesFrames[profilesName][0]
            else:
                dNewProfilesFrames[profilesName] = 0
        return dNewProfilesFrames

    def _isOTHERNotInSeqName(self, seqname):
        return not re.search('_OTHER_', seqname)