from PASTEC.utils.IAgent import IAgent
from PASTEC.utils.AgentFactory import AgentFactory
from PASTEC.Demand import Demand
from PASTEC.ClassifierResult import ClassifierResult

class AbstractClassifierAdaptatorAgent(IAgent):
    
    ## Set ConsensusName
    #    
    def setConsensus(self, consensusName):
        self._consensusName = consensusName 
        
    ## Accept or not a demand
    #
    # @param demand a Demand object type (name of Agent and name of sequence to analyse for example)
    # @return True if demand is accepted, false otherwise
    #
    def acceptDemand(self, demand):
        return demand.agent == self.__class__.__name__\
            and not self._environment.isDemandOnBlackBoardResults(demand)
            
    ## Get and filter analysis if corresponding table is in database.
    #
    def getAnalysesResults(self):
        for iAgent in self._dAgentName2AnalyserInstance.values():
            iAgentResult = self._callAgent(iAgent)
            method2Call = "self.makeAppropriateConclusionsFor" + str(iAgent.__class__.__name__) + "( iAgentResult )"
            exec ( method2Call )
            
    ## Three main step : make demands, get results and decide
    #
    def live(self):
        for demand in self._environment.getBlackBoardDemands():
            if self.acceptDemand(demand):
                self._consensusName = demand.consensusName 
                self._initializeConclusions()
                self._makeDemands()               

                self.getAnalysesResults()
                classification, normalizedIC = self.decide()
                message = self.generateMessage(classification, normalizedIC)
                
                finalResult = ClassifierResult(demand, message)
                self._environment.addResultToBlackBoard(finalResult)
                    
    ## Create Needed Agent instances if corresponding table is in database.
    # _dAgentName2AnalyserInstance is filled with analyzer agent class name as key
    # and associated instance as value
    #
    def createNeededAgentInstance(self):
        for knownAgentName in self._listOfKnownAgents:
            self._createOneNeededAgentInstance(knownAgentName)
    
    def _createOneNeededAgentInstance(self, agentName):
        currentAgentNewInstance = AgentFactory.createAgent(agentName, self._environment)
        lTableSuffix = currentAgentNewInstance.getSuffixTableNameList()
        isCreateInstance = True
        for tableSuffix in lTableSuffix:
            tableName = self._environment.getProjectName() + tableSuffix
            if tableName not in self._environment.getTablesList():
                isCreateInstance = False
                break
        if isCreateInstance:
            self._dAgentName2AnalyserInstance [currentAgentNewInstance.__class__.__name__] = currentAgentNewInstance
            currentAgentNewInstance.createNeededAgentInstance()
        
    def _callAgent (self, AgentInstance):
        AgentInstance.createNeededAgentInstance()
        AgentInstance.live()
        iDemand = Demand(AgentInstance.__class__.__name__, self._consensusName)
        try:
            iDemandAndResult = self._environment.getResultAccordingToDemand(iDemand)
        except:
            return None
        return AgentInstance.patch(iDemandAndResult)
    
    def _makeDemands(self):
        for agentName in self._dAgentName2AnalyserInstance.keys():
            iDemand = Demand(agentName, self._consensusName)
            try:
                #TODO: ??? getResultAccordingToDemand() return a result if exists. Why call this method ? Why "try/except" ?
                self._environment.getResultAccordingToDemand(iDemand)
            except Exception:
                if not self._environment.isDemandOnBlackBoardDemands(iDemand):
                    self._environment.addDemandToBlackBoard(iDemand)
                    
    def _normalizeIC(self):
        normalizedIC = 0
        if not self._CI <= 0:
            normalizedIC = (self._CI * self._environment.getMaxCI()) / self.CI_MAX
            if normalizedIC > self._environment.getMaxCI():
                normalizedIC = self._environment.getMaxCI()
        return normalizedIC
    
    def _initializeConclusions(self):
        pass
    
    def decide(self):
        pass
    
    def generateMessage(self, classification, IC):
        pass
    
    def _fillDictOfEvidence(self):
        pass