from PASTEC.utils.AbstractClassifierAdaptatorAgent import AbstractClassifierAdaptatorAgent
import re

class AbstractClassifierAgent(AbstractClassifierAdaptatorAgent):
    
    CI_MAX = 1
    INIT_CLASSIF = "!"
    SINGLE_CONDITIONS = ()
    COMPLEXE_CONDITIONS = ()
    PATTERN_2_SEARCH_IN_BLAST_RESULTS = ""
    #TODO: NOPATTERN_2_SEARCH_IN_BLAST_RESULTS to remove, when repbase with cleaned headers will be always used
    NOPATTERN_2_SEARCH_IN_BLAST_RESULTS = ">"
    PROFILES_2_SEARCH_IN_ORF = ()
    INCREASED_CI_CONDITIONS = []
    DECREASED_CI_CONDITIONS = []
    CATEGORY_LENGTH_BOUND = []

    ## Constructor
    #
    # @param env Environment instance
    # @param projectName string project name
    #
    def __init__(self, env):
        self._environment = env
        self._dAgentName2AnalyserInstance = {}
        self._listOfKnownAgents = ["Size_Agent", "TRSearch_Agent", "TE_BLRx_Agent", "TE_BLRtx_Agent", "TE_Profiles_Agent", "ORF_Agent"]
        self._CI = 0
        
    def getCI(self):
        return self._CI
        
    def getConclusionsSearch(self):
        return self._conclusionsSearch
    
    def getConclusionsMatch(self):
        return self._conclusionsMatch
    
    ## Set conclusionsSearch
    #  
    def setConclusionsSearch(self, conclusionsSearch):
        self._conclusionsSearch = conclusionsSearch

    ## Set conclusionsMatch
    #  
    def setConclusionsMatch(self, conclusionsMatch):
        self._conclusionsMatch = conclusionsMatch
    
    ## Give the classification and the confidence index from all analysis conclusions
    #
    # @return classification, normalized confidence index
    #
    def decide(self):
        classification = self.INIT_CLASSIF
        self._CI = 0
        classification = self._decideForConclusionsMatch(classification)
        classification = self._decideForConclusionsSearch(classification)
        classification = self._decideForComplexConditions(classification)
        normalizedIC = self._normalizeIC()
        return classification, normalizedIC
    
    ## Create the appropriate message of classification, confidence index,consensus name and the dict of evidence
    #
    # @param classification string 
    # @param IC integer confidence index
    #
    # @return dict of message
    #
    def generateMessage(self, classification, IC):
        if classification == self.INIT_CLASSIF:
            IC = self._environment.getMaxCI()
        dictOfEvidence = self._fillDictOfEvidence(self.PATTERN_2_SEARCH_IN_BLAST_RESULTS)
        message = {'consensusName': self._consensusName, 'classif': classification, 'confidenceIndex': IC, 'dictOfEvidence': dictOfEvidence}
        return message
                
    def makeAppropriateConclusionsForSize_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != None:
            iSize = iDemandAndResult.result
            consensusLengthWithoutTR = iSize.getSizeWithoutTR()
            lBound = sorted(self.CATEGORY_LENGTH_BOUND)
            msg = "<%sbps" % lBound[0]
            for bound in lBound :
                if consensusLengthWithoutTR > bound:
                    msg = ">%sbps" % bound
            self._conclusionsSearch.append(msg)
                            
    def makeAppropriateConclusionsForTE_BLRx_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != []:
            self._conclusionsMatch.append({'Repbase_bx': iDemandAndResult.result})

    def makeAppropriateConclusionsForTE_BLRtx_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != []:
            self._conclusionsMatch.append({'Repbase_tbx': iDemandAndResult.result})

    def makeAppropriateConclusionsForTE_Profiles_Agent(self, iProfilesResult):
        if iProfilesResult != None:
            self._conclusionsMatch.extend(iProfilesResult.listProfilesDetailed)

    def makeAppropriateConclusionsForORF_Agent(self, iORF_AgentResult):
        if iORF_AgentResult != None and not (iORF_AgentResult.isEmpty()) :
            self._conclusionsMatch.append({'ORF': iORF_AgentResult.ORF_List})

    def makeAppropriateConclusionsForTRSearch_Agent(self, iTRSearchResult):
        pass

    def considerMultipleConclusionIfTheyAreDiscriminant(self, tuple, classification):
        dNbClassifProofFound = {}
        if len(tuple) < 2:
            return classification
        for item in tuple:
            dNbClassifProofFound[item] = self._getNbOfValidOccInConclusionSearch(item)
            if dNbClassifProofFound[item] == 0 :
                dNbClassifProofFound[item] = self._getNbOccInConclusionMatch(item)
        increaseOfClassif = 1
        for key in dNbClassifProofFound :
            increaseOfClassif = increaseOfClassif * dNbClassifProofFound[key]
        if increaseOfClassif > 0:
            classification = self.getAgentName()
            self._CI += increaseOfClassif
        return classification

    def _increaseIC(self, conclusion, classification):
        for itemOfCondition in self.INCREASED_CI_CONDITIONS:
            if self._isEvidencesEqualOrRealSizeCategoryIsValidComparedToConditionSizeCategory(conclusion, itemOfCondition):
                self._CI += 1
                
    def _decreaseIC(self, conclusion, classification):
        for itemOfCondition in self.DECREASED_CI_CONDITIONS:
            if self._isEvidencesEqualOrRealSizeCategoryIsValidComparedToConditionSizeCategory(conclusion, itemOfCondition):
                self._CI = self._CI - 1

    def _isEvidencesEqualOrRealSizeCategoryIsValidComparedToConditionSizeCategory(self, conclusion, itemOfCondition):
        motifSize = '^[><][0-9]+bps$'
        isCategoryValid = False
        if re.match(itemOfCondition, str(conclusion)):
            isCategoryValid = True
        elif re.match(motifSize, itemOfCondition) and re.match(motifSize, str(conclusion)):
            conditionBound = int(itemOfCondition[1:][:-3])
            realBound = int(conclusion[1:][:-3])
            if itemOfCondition[0] == "<":
            #NOTE: not valid, because the smallest category is always with "<" and the only one with "<".
            #    So valid case is in first "if", otherwise all real categories should be greater. 
                isCategoryValid = False
            elif realBound > conditionBound:
                isCategoryValid = True
        return isCategoryValid
    
    def _fillDictOfEvidence(self, motif2Search):
        dictOfEvidence = {}
        listOfTR = []
        listOfSSRtrf = []
        dictOfHelitronsExtremities = {}
        dictOfProfiles = {}
        for iConclusion in self._conclusionsSearch:
            strConclusion = str(iConclusion)
            if re.match(".*term.*", strConclusion):
                listOfTR.append(iConclusion)
            if re.match("polyAtail", strConclusion):
                dictOfEvidence['polyAtail'] = "yes"
            if re.search("_start|_end", strConclusion):
                listOfSSRtrf.append(iConclusion)
            if re.match(".*bps", strConclusion):
                dictOfEvidence['length'] = iConclusion
        for conclusion in self._conclusionsMatch:
            if conclusion.has_key('Repbase_tbx'):
                lBM = []
                for iBlasterMatch in conclusion['Repbase_tbx']:
                    if re.search(motif2Search, iBlasterMatch.getMatchName()):
                        lBM.append(iBlasterMatch)
                if lBM != []:
                    dictOfEvidence['Repbase_tbx'] = lBM
            if conclusion.has_key('Repbase_bx'):
                lBM = []
                for iBlasterMatch in conclusion['Repbase_bx']:
                    if re.search(motif2Search, iBlasterMatch.getMatchName()):
                        lBM.append(iBlasterMatch)
                if lBM != []:
                    dictOfEvidence['Repbase_bx'] = lBM
            if conclusion.has_key('name'):
                dictOfProfiles[conclusion['name']] = conclusion['coverage']
            if conclusion.has_key('ORF'):
                dictOfEvidence['ORF'] = conclusion['ORF']
            if conclusion.has_key('helitronExtremities'):
                dictOfHelitronsExtremities = conclusion['helitronExtremities']
        if listOfTR != []:
            dictOfEvidence['TR'] = listOfTR
        if listOfSSRtrf != []:
            dictOfEvidence['SSRtrf'] = listOfSSRtrf
        if dictOfProfiles != {}:
            dictOfEvidence['te_hmmer'] = dictOfProfiles
        if dictOfHelitronsExtremities != {}:
            dictOfEvidence['helitronExtremities'] = dictOfHelitronsExtremities
        return dictOfEvidence
    
    def _considerProfilesInSameFrameAsORF(self, conclusion):
        if conclusion.has_key('ORF'):
            lType2SearchInORF = self.PROFILES_2_SEARCH_IN_ORF
            for item in conclusion['ORF']:
                lTypeInORF = item.split(" ")
                for type in lTypeInORF[1:]:
                    if type in lType2SearchInORF:
                        self._CI += 1
                
    def _considerBlastxAndTBlastxEvidence(self, conclusion, classification):
        for lBM in conclusion.values():
            for iBlasterMatch in lBM:
                #TODO: re.search(..., ..., re.IGNORECASE) ? change pattern in classifier agents ?
                if re.search(self.PATTERN_2_SEARCH_IN_BLAST_RESULTS, iBlasterMatch.getMatchName()) \
                and not re.search(self.NOPATTERN_2_SEARCH_IN_BLAST_RESULTS, iBlasterMatch.getMatchName()):
                    classification = self.getAgentName()
                    self._CI += 2
                    break
        return classification
    
    def _considerProfilesIfTheyAreDiscriminant(self, conclusion, classification, lType):
        for evidence in self.SINGLE_CONDITIONS:
            if conclusion.has_key('type') and re.match("^"+evidence+"$", conclusion['type']):
                classification = self.getAgentName()
                if not conclusion['type'] in lType: 
                    self._CI += 2
                lType.append(conclusion['type'])
        return classification, lType

    def _considerConclusionIfItIsDiscriminant(self, conclusion, classification):
        for evidence in self.SINGLE_CONDITIONS:
            if re.match(evidence, str(conclusion)):
                classification = self.getAgentName()
                self._CI += 2
        return classification
    
    def _getNbOccInConclusionMatch(self, item) :
        dConclusionFound = {}
        for conclusion in self._conclusionsMatch:
            if conclusion.has_key('type') and re.match(item, conclusion['type']): 
                dConclusionFound[conclusion['type']] = 1
        return len(dConclusionFound)
                        
    def _getNbOfValidOccInConclusionSearch(self, itemOfCondition):
        nbOccur = 0
        for conclusion in self._conclusionsSearch:
            if self._isEvidencesEqualOrRealSizeCategoryIsValidComparedToConditionSizeCategory(conclusion, itemOfCondition):
                nbOccur += 1
        return nbOccur
                
    def _decideForConclusionsMatch(self, classification):
        lType = []
        for conclusion in self._conclusionsMatch:
            if conclusion.has_key('Repbase_tbx') or conclusion.has_key('Repbase_bx'):
                classification = self._considerBlastxAndTBlastxEvidence(conclusion, classification)
            elif conclusion.has_key("ORF"):
                self._considerProfilesInSameFrameAsORF(conclusion)
            else:
                classification, lType = self._considerProfilesIfTheyAreDiscriminant(conclusion, classification, lType)
        return classification

    def _decideForConclusionsSearch(self, classification):
        for conclusion in self._conclusionsSearch:
            classification = self._considerConclusionIfItIsDiscriminant(conclusion, classification)
            self._increaseIC(conclusion, classification)
            self._decreaseIC(conclusion, classification)
        return classification

    def _decideForComplexConditions(self, classification):
        for tuple in self.COMPLEXE_CONDITIONS:
            classification = self.considerMultipleConclusionIfTheyAreDiscriminant(tuple, classification)
        return classification
    
    def _initializeConclusions(self):
        self._conclusionsSearch = []
        self._conclusionsMatch = []